@@ -4,6 +4,10 @@ module Data.GI.GIR.Object
     , parseObject
     ) where
 
+#if !MIN_VERSION_base(4,13,0)
+import Data.Monoid ((<>))
+#endif
+
 import Data.Text (Text)
 
 import Data.GI.GIR.Method (Method, parseMethod, MethodType(..))
@@ -19,6 +23,8 @@ data Object = Object {
     objCType :: Maybe Text,
     objRefFunc :: Maybe Text,
     objUnrefFunc :: Maybe Text,
+    objSetValueFunc :: Maybe Text,
+    objGetValueFunc :: Maybe Text,
     objInterfaces :: [Name],
     objDeprecated :: Maybe DeprecationInfo,
     objDocumentation :: Documentation,
@@ -46,6 +52,8 @@ parseObject = do
   signals <- parseChildrenWithNSName GLibGIRNS "signal" parseSignal
   refFunc <- queryAttrWithNamespace GLibGIRNS "ref-func"
   unrefFunc <- queryAttrWithNamespace GLibGIRNS "unref-func"
+  setValueFunc <- queryAttrWithNamespace GLibGIRNS "set-value-func"
+  getValueFunc <- queryAttrWithNamespace GLibGIRNS "get-value-func"
 
   ctype <- queryCType
   return (name,
@@ -55,6 +63,8 @@ parseObject = do
           , objCType = ctype
           , objRefFunc = refFunc
           , objUnrefFunc = unrefFunc
+          , objSetValueFunc = setValueFunc
+          , objGetValueFunc = getValueFunc
           , objTypeName = typeName
           , objInterfaces = interfaces
           , objDeprecated = deprecated
